      .286

CODE  SEGMENT BYTE PUBLIC
      ASSUME CS:CODE

;*****************************************************************************
;
; Steves VESA Video Driver based on original protel CGA.ASM / VEGA.ASM
;
; Written By  : Steve Murray, steve@airborn.com.au
; Original    : N.M.Martin, (Protel) 21-June-1988
; Updates at  : http://www.airborn.com.au/layout/easytrax.html
; Last Change : 18-Mar-2013
;               Modify to remove VGA bios call for background colour setting
;*****************************************************************************
;12 Sept 00 Small problem: XOR'ed colours were moving outside Pallette: Limited
;18 Sept 00 Replaced screen word write with exclusively byte Write operations
;           Fudged Hpixel count to Mimic orig protel HLINE start/end point V1.02
;19 Sept 00 Windowing problem on ATI Rage 3D: workaround, set both win 1&2 V1.03
;20 Sept 00 Redoing ClearGraph and adding a couple of VESA CALL fail safes V1.04
;26 Sept 00 One off for Jamie Rogers - force mode setting on each redraw
;28 Dec  01 Adding Black background colour, replaces colour 0 with val 16  V1.10
;           Modification original code and test by DW Coates dwcoates@nwlink.com
;29 Jan  02 Pulling in the ForceMode setting on every redraw as asm option V1.11
;21 May  02 Add improved Arc draw (in gcommon2) by Richard Aston           V1.12
;13 Oct  04 Changing Pallette VGA calls to VESA bios calls if version is >= VBE2
;18 Mar  13 Adding some more VESA modes, after best effort research for mode Num

public    GETDRIVERTYPE ;In 'Common' - Returns 0 for Graphic
public    DRAW          ;In 'Common' - Bresenhams line Drawing algorithm
public    HLINE         ;HW specific - Draw horizontal line
public    VLINE         ;HW specific - Draw vertical line
public    PLOTPIXEL     ;In 'Common'
public    READPIXEL     ;HW specific - Have general VESA routine for 8bit colour
public    CLEARGRAPH    ;HW specific - rewritten here as general case for VESA
public    GRAPHMODE     ;HW specific - we just ask for mode and expect to get it
public    TEXTMODE1     ;HW specific - can't see a good reason to change Orignal
public    FULLCHAR      ;In 'Common'
public    FASTCHAR      ;In 'Common'
public    SAVEMENU      ;HW specific - Have general VESA routine, saves 4bit col
public    LOADMENU      ;HW specific - Have general VESA routine, restr 4bit col
public    XBIG          ;In 'Common' MaxX-1 returned      e.g. 800
public    YBIG          ;In 'Common' MaxY-1 returned      e.g. 600
public    BIGCOLORS     ;In 'Common' MaxColours returned        15
public    BIGBACKCOLORS ;In 'Common' MaxBackColours returned    15
public    COLORSETS     ;In 'Common' MaxSets returned            0
public    ARROWCURSOR   ;HW specific - Original code works fine
public    SETBACKGROUND ;HW specific - Have general VESA routine for 8bit colour
public    MENUMEMREQUIRED ; 'Common' Menumem value returned  32000
public    RAWPLOT       ;In 'Common'
public    ARC           ;In 'Common' Bresenhams circle drawing
public    GETDRIVERNAME ;Location of driver ID string in File
public    RETURNECX     ;Return address after ECX

;*****************************************************************************
; This Table is the external reference point for this driver. The order and
; number of elements in it must not be changed or a program crash will result.
;*****************************************************************************

GETDRIVERTYPE:   Jmp    _GETDRIVERTYPE
DRAW:            Jmp    _DRAW
HLINE:           Jmp    _HLINE
VLINE:           Jmp    _VLINE
PLOTPIXEL:       Jmp    _PLOTPIXEL
READPIXEL:       Jmp    _READPIXEL
CLEARGRAPH:      Jmp    _CLEARGRAPH
GRAPHMODE:       Jmp    _GRAPHMODE
TEXTMODE1:       Jmp    _TEXTMODE
FULLCHAR:        Jmp    _FULLCHAR
FASTCHAR:        Jmp    _FASTCHAR
SAVEMENU:        Jmp    _SAVEMENU
LOADMENU:        Jmp    _LOADMENU
XBIG:            Jmp    _XBIG
YBIG:            Jmp    _YBIG
BIGCOLORS:       Jmp    _BIGCOLORS
BIGBACKCOLORS:   jmp    _BIGBACKCOLORS
COLORSETS:       Jmp    _COLORSETS
ARROWCURSOR:     Jmp    _ARROWCURSOR
SETBACKGROUND:   Jmp    _SETBACKGROUND
MENUMEMREQUIRED: Jmp    _MENUMEMREQUIRED
RAWPLOT:         Jmp    _RAWPLOT
ARC:             Jmp    _ARC

;*****************************************************************************
; The Driver name below must remain in the same position and have the same   ;
; length between then quotes.                                                ;
;*****************************************************************************
;Video modes supported '|----title-should-fit-here-----|' Uncomment 1 line only:
;ETDRIVERNAME:  DB  32,'VESA mode 101h: 640  x 480 V1.14' ;VESA640.DRV
;ETDRIVERNAME:  DB  32,'VESA mode 103h: 800  x 600 V1.14' ;VESA800.DRV
;ETDRIVERNAME:  DB  32,'VESA mode 16Ah: 1024 x 640 V1.14' ;V1024640.DRV
;ETDRIVERNAME:  DB  32,'VESA mode 105h: 1024 x 768 V1.14' ;VESA1024.DRV
;ETDRIVERNAME:  DB  32,'VESA mode 165h: 1152 x 720 V1.14' ;V1152720.DRV
GETDRIVERNAME:  DB  32,'VESA mode 207h: 1152 x 864 V1.14' ;VESA1152.DRV
;ETDRIVERNAME:  DB  32,'VESA mode 160h: 1280 X 800 V1.14' ;V1280800.DRV
;ETDRIVERNAME:  DB  32,'VESA mode 107h:1280 x 1024 V1.14' ;VESA1280.DRV
;ETDRIVERNAME:  DB  32,'VESA mode 183h: 1366 X 768 V1.14' ;VESA1366.DRV
;ETDRIVERNAME:  DB  32,'VESA mode 160h: 1440 X 900 V1.14' ;VESA1440.DRV
;ETDRIVERNAME:  DB  32,'VESA mode 143h:1400 x 1050 V1.14' ;VESA1400.DRV
;ETDRIVERNAME:  DB  32,'VESA mode 11Ch:1600 x 1200 V1.14' ;VESA1600.DRV
;ETDRIVERNAME:  DB  32,'VESA mode 166h:1920 x 1080 V1.14' ;VESA1920.DRV
;ETDRIVERNAME:  DB  32,'VESA mode 17Ch:1920 x 1200 V1.14' ;V1920X12.DRV
;ETDRIVERNAME:  DB  32,'VESA 640x480 with ModeforceV1.14' ;V640MDF.DRV Mode 101h
;ETDRIVERNAME:  DB  32,'VESA 800x600 with ModeforceV1.14' ;V800MDF.DRV Mode 103h
;ETDRIVERNAME:  DB  32,'VESA1024x768 with ModeforceV1.14' ;V1024MDF.DRV Mode105h
;ETDRIVERNAME:  DB  32,'VESA1280x1024with ModeforceV1.14' ;V1280MDF.DRVMode 107h
;ETDRIVERNAME:  DB  32,'VESA1600x1200with ModeforceV1.14' ;V1600MDF.DRVMode 120h
;ETDRIVERNAME:  DB  32,'Experiment103h: 800 x  600 V1.14'

                               ;Take the 3 parameters listed above,repeat below:
VidMode       Equ 207h         ;Desired VESA video mode matching these params
MaxX          Equ 1152         ;Number of Pix in X Direction
MaxY          Equ 864          ;Number of Pix in Y Direction
modeforce     Equ 0            ;1 - assemble with modeforce, 0 assemble without
                               ;Modeforce really impacts performance - only use
                               ; if really needed - for 0S2 or problem vidcards

MaxColors     Equ 15           ;Largest Color Number that driver supports(2^n-1)
MaxBackColors Equ 15           ;Largest Background Color the driver supports
MaxSets       Equ 0            ;Number of available color sets
MenuMem       Equ 32000        ;Mem space to save a screen menu- (protel < 32k)
StatusLine    Equ 23           ;Number of Y Pix Reserved For Status Line
Membase       Equ 0A000h       ;General location for VESA video memory buffer

INCLUDE GCOMMON2.ASM  ;These call: SetGraph,ResetGraph,Point,TextPoint ONLY
;Gets _PLOTPIXEL      calls SetGraph,ResetGraph,Point,
;     _RAWPLOT        calls SetGraph,ResetGraph,Point,
;     _DRAW           calls SetGraph,ResetGraph,Point,
;     _FASTCHAR       calls SetGraph,ResetGraph,Point,
;     _FULLCHAR       calls SetGraph,ResetGraph,      TextPoint,
;     _GETDRIVERTYPE                                             returns Value
;     _BIGCOLORS                                                 returns Value
;     _BIGBACKCOLORS                                             returns Value
;     _XBIG                                                      returns Value
;     _YBIG                                                      returns Value
;     _MENUMEMREQUIRED                                           returns Value
;     _COLORSETS                                                 returns Value
;     _Arc            calls SetGraph,ResetGraph,Point

;These routines are common for all graphics drivers (updated 21 May 2002)

        db      0
;*************************************************;
SetGraph Proc Near
; This routines sets up the graphics system for
; drawing. It sets the segment registers
;*************************************************;
        push    bx
         call   OurCod6         ;Find our current location
OurCod6: pop    bx
        sub  bx,offset(OurCod6) ;bx = point to start of section
        mov  bx,cs:WinSegA[bx]  ;Get the memory window segment
        push    bx              ;Save the segment specified
        and     bx,0E000h       ;Mask off lower 13 bits of segment
        cmp     bx,0A000h       ;Test if segment is 0A000h-0BFFFh
        je      SetGr1          ; and jump to accept value
        pop     bx              ;Discard the segment specified
        mov     bx,Membase      ;Not happy with segment specified - use our own
        push    bx

SetGr1: pop     bx
        mov     es,bx
        pop     bx
        ret
SetGraph Endp

;*********************************************;
ResetGraph proc Near                          ;
; Resets Graphics hardware to state where an  ;
; exit from the program would be acceptable.  ;
; The Driver should always leave the hardware ;
; in such a state.                            ;
;*********************************************;
        ret
ResetGraph endp


;********************************************;
_READPIXEL Proc NEAR                         ;
; Return the color of the pixel at (Rpx,Rpy) ;
; X = Cx                                     ;
; Y = Dx                                     ;
; Return Ax = color                          ;
;********************************************;
Rpx     equ     [bp+6]
Rpy     equ     [bp+4]
        push    bp
        push    bx
        mov     bp,sp

        call    SetGraph

        mov     dx,rpy          ;y
        mov     cx,rpx          ;x

        call    PixelPos        ;Address the pixel
        mov     al,es:0[bx]     ;Get the data
        mov     ah,0            ;Set top byte of data to 0

        pop     bx
        push    ax              ;Return the data on stack and in ax
        pop     ax
        pop     bp
        ret     6
_READPIXEL endp


;*************************************;
_CLEARGRAPH proc NEAR                 ;
; Set all pixels on the screen to the ;
; background color.                   ;
;*************************************;

IF modeforce
;----                                   ;Alternative number one:
       mov     ax,4F02h                ;Horrible crass video clear - set mode
       mov     bx,VidMode              ;Set appropriate video mode
       int     10h
       call    SetGraph
       ret
;----
ENDIF                  
                                       ;Alternative number two - much nicer
        call    SetGraph

        mov     dx,(MaxY-1)             ;Get the last scan line in dx

ClrG1:  mov     cx,(MaxX-1)             ;Last pixel on scan line to cx
ClrG2:  call    PixelPos                ;Find pixel address of cx:dx
ClrG3:  mov     al,0
        mov     es:0[bx],al             ;Clear the pixel

        sub     cx,1                    ;Move one pixel position to the left
        jb      ClrG4                   ;Jump out if we have finished a line

        sub     bx,1                    ;Move address down one location
        jnb     ClrG3           ;If we have not wrapped the video page, continue
        jmp     ClrG2           ;  else re-calculate the video page address

ClrG4:  sub     dx,1                    ;Move pixel position one line upwards
        jnb     ClrG1                   ;If not finished screen, loop until done

        ret
_CLEARGRAPH endp

;*********************************************
_TEXTMODE proc NEAR                          ;
; Places the graphics hardware into textmode ;
;*********************************************
      mov    ax,0003h
      int    10h
      ret
_TEXTMODE endp

;**********************************************;
_GRAPHMODE proc NEAR                           ;
; Places the graphics hardware into Graph Mode ;
;**********************************************;

        call    SetupForLEA     ;Find offset, load segment to ES
        lea     di,VBEInfoBlock ;Get the address of the Vesa buffer
        add     di,bx           ;  and fix up the offset
                                ;es:di now point to VBEInfoBlock

        mov     ax,4F00h        ;Detect if VESA mode exists
        int     10h
        cmp     ax,4Fh          ;Test if the function was successful
        jne     GrIniFail       ;If unsuccessful, try indicating the problem

        call    Capabilities    ;Get (and save) VESA version number and VGA bits
        and     ax,0F300h       ;Mask to see just the version number
        cmp     ax,300h         ;Look for version 3 or better
        mov     bx,VidMode      ;Set appropriate video mode for init
        jl      PastRefresh     ;Jump past refresh if not > VESA BIOS Version 3

        call    SetupForLEA     ;Find offset, load segment (to es and cs)
        lea     di,PixelClock   ;Get the address of the PixelClock parameter
        add     bx,di           ;  and fix up the offset
        mov     ax,4F0Bh        ;Get/Set pixel clock
        mov     bl,0            ;Get closest pixel clock
        mov     dx,VidMode      ;Set mode number pixel clock to be used with

     mov cx,es:PixelClock+2[bx] ;Get the Pixel clock high value
        push    cx
     mov cx,es:PixelClock+0[bx] ;Get the Pixel clock low value
        push    cx

        jmp     MOVEECX         ;Take these values, put them in ECX
RETURNECX:

        call    SetupForLEA     ;Find offset, load segment (to es and cs)
        lea     di,PixelClock   ;Get the address of the PixelClock parameter
        add     bx,di           ;  and fix up the offset

        pop     cx
     mov es:PixelClock+0[bx],cx ;Save the Pixel clock low value
        pop     cx
     mov es:PixelClock+2[bx],cx ;Save the Pixel clock high value

        call    GTFGen          ;Fill in the parameter tables for the CRTC
LeaveOutVBE3:
        call    SetupForLEA     ;Find offset, load segment to ES
        lea    di,CRTCInfoBlock ;Get the address of the Vesa buffer
        add     di,bx           ;  and fix up the offset
                                ;es:di now point to CRTInfoBlock

        mov     bx,VidMode      ;Set appropriate video mode
        add     bx,2048         ;Set bit 11 to ask for forced video parameters
PastRefresh:
        mov     ax,4F02h
        int     10h
        cmp     ax,4Fh          ;Test if the function was successful
        jne     GrIniFail       ;If unsuccessful, try indicating the problem

                                ;Success...
        call    SetupForLEA     ;Find offset, load segment to ES
        lea     di,VesaBuf      ;Get the address of the Vesa buffer
        add     di,bx           ;  and fix up the offset

                                ;es:di now points to Vesa buffer
        mov     cx,VidMode      ;Ask for the same video mode again
        mov     ax,4F01h        ;Get the buffer filled with the information
        int     10h
        cmp     ax,4Fh          ;Test if the function was successful
        jne     GrIniFail       ;If unsuccessful, try indicating the problem

        call    SetGraph
        ret

GrIniFail:
        call    _TEXTMODE       ;Go to text mode

         call   OurCod4         ;Find our current location
OurCod4: pop    bx
         sub bx,offset(OurCod4) ;bx = point to start of section

        mov     ax,0            ;Restore parameter area to safe defaults
        mov  es:PageFun[bx],0   ;Overwrite Page function with null pointer
        mov  es:PageFun+2[bx],0 ;  ..second word
        mov es:ByPerLL[bx],MaxX ;And bytes per line
     mov es:WinSegA[bx],Membase ;And memory segment
     mov es:CurrWin[bx],0FEDCh  ;And current window
        mov es:WinGran[bx],64   ;And window granularity

        call    Capabilities    ;Get the VESA version number and VGA bits
        mov     al,ah           ;Get the VESA bios version
        and     al,0Fh          ;Restrict to one nybble
        add     al,30h          ;Offset to make it 0-9 character
       mov es:FailMsg+25[bx],al ;Put the version number in the failure message

        mov     cx,69           ;Failure message part 1 is 69 characters long
GrapM3: mov   al,es:FailMsg[bx] ;Get the character to print
        push    bx
        push    cx
        mov     bx,0Fh          ;Print it in colour white (appl Gr mode only)
        mov     ah,0Eh          ;BIOS call func 0Eh - write char in teletype
        int     10h             ;Send char to screen
        pop     cx
        pop     bx
        inc     bx
        loop    GrapM3          ;Loop until the failure message is sent
        sub     bx,69           ;Get bx back to index value
        mov     cx,33           ;Driver name is 33 characters long
GrapM4:
    mov al,es:GETDRIVERNAME[bx] ;Get the character to print
        push    bx
        push    cx
        mov     bx,0Fh          ;Print it in colour white (appl Gr mode only)
        mov     ah,0Eh          ;BIOS call func 0Eh - write char in teletype
        int     10h             ;Send char to screen
        pop     cx
        pop     bx
        inc     bx
        loop    GrapM4          ;Loop until the failure message is sent

        mov     dx,0
        mov     cx,2*30         ;Wait for 2 seconds (cx:dx in us)
        mov     ax,8600h        ;Function call value for wait operation
        int     15h             ;Perform the wait - allow message to be seen

        call    SetGraph        ;Leave es set to correct value in any case
        ret                     ;Failure...
_GRAPHMODE endp


;*****************************************
Capabilities Proc Near
; Sub_Routine to report capabilities
; ax = capabilities
;   eg, 0301h = VBE3.0, no VGA support
;  0202h = VBE2.0, DAC can switch to 8 bit
;*****************************************
        push    bx
        push    cx

        call   OurCod8          ;Find our current location
OurCod8: pop    bx
        sub bx,offset(OurCod8)  ;bx = point to start of section
                                ;Example code to use our routine:

        mov ax,cs:Capability[bx] ;Get the capabilities
        add     ax,0             ;Test the result
        jne     HaveCapable      ;Jump if we have save capabilities values

        mov   ax,cs:VBEVers[bx] ;Get the version to ax

        mov cx,cs:VBEVers+2[bx] ;Get the capabilities to bx
        and     cx,3            ;Only interested in DAC/VGA capabilites
        and     ax,0FFF0h       ;Mask out LSD of version
        add     ax,cx           ;Combine DAC/VGA capabilities into version

        mov cs:Capability[bx],ax ;Save the capabilities for the next call
HaveCapable:
        pop     cx
        pop     bx
        ret
Capabilities endp


;*****************************************
SetupForLEA  Proc Near
; Sets ES to CS, exits with offset in BX
;*****************************************
         call   OurCod9         ;Find our current location
OurCod9: pop    bx
         sub bx,offset(OurCod9) ;bx = point to start of section
        push    cs              ;Copy code segment
        pop     es              ;  into es
        ret
SetupForLEA  endp


;*********************************************;
_SAVEMENU proc NEAR                           ;
; This routine saves the area of the top left ;
; hand of the screen used to store data under ;
; the pop-up menu.                            ;
; The area is 320 pixels horizontal and 200   ;
; pixels vertical                             ;
; We would love to save byte values, but      ;
; protel seems to gag if we ask for more than ;
; 32k of memory for the save buffer...        :
; StoreOfs and storeSeg point to an area of   ;
; memory where the screen data can be stored. ;
;*********************************************;
storeOfs equ [bp+6]
storeSeg equ [bp+8]
        push    ds
        push    bp
        mov     bp,sp
        push    es
        cld

        call    SetGraph        ;Find our graphics segment
        mov     ax,es
        mov     ds,ax           ;Put to ds

        mov     di,StoreOfs     ;di = offset to store data
        mov     ax,StoreSeg
        mov     es,ax           ;es = Seg to store data

        mov     dx,0            ;Start at y=0
SavMn2: mov     cx,0            ;Start at x=0

SavMn1: call    PixelPos        ;Find the position of the pixel
        mov     si,bx           ;Get the index of the pixel to read

        mov     al,ds:[si]      ;Get left screen pixel
        inc     si              ;  bump up pointer
        sal     al,4            ;Put it in the high nybble

        mov     ah,ds:[si]      ;Get right screen pixel
        inc     si              ;  bump up pointer
        and     ah,0Fh          ;Mask to leave just low nybble
        or      al,ah           ;Pack the two pixels into one byte

        mov     es:[di],al      ;Save the packed data to the buffer
        inc     di              ;  bump up pointer

        mov     al,ds:[si]      ;Get left screen pixel
        inc     si              ;  bump up pointer
        sal     al,4            ;Put it in the high nybble

        mov     ah,ds:[si]      ;Get right screen pixel
        and     ah,0Fh          ;Mask to leave just low nybble
        or      al,ah           ;Pack the two pixels into one byte

        mov     es:[di],al      ;Save the packed data to the buffer
        inc     di              ;  bump up pointer

        add     cx,4            ;update 'x'
        cmp     cx,320          ;Test if we have finished one line yet
        jne     SavMn1          ;Loop until a line is finished

        inc     dx              ;We have transferred one line, update 'y'
        cmp     dx,200          ;Test if we have finished the required 200 lines
        jne     SavMn2          ;Loop until the whole transfer is complete

        pop     es
        pop     bp
        pop     ds
        ret     4
_SAVEMENU endp

;***********************************************;
_LOADMENU proc NEAR                             ;
; This routine retrives the stored screen data  ;
; and places in into the area of the top left   ;
; hand of the screen.                           ;
; The area is 320 pixels horizontal and 200     ;
; pixels vertical                               ;
; LoadOfs and LoadSeg point to the area in mem  ;
; where the data was stashed.                   ;
;***********************************************;
LoadOfs equ [bp+6]
LoadSeg equ [bp+8]

        push    ds
        push    bp
        mov     bp,sp
        push    es
        cld

        mov     si,LoadOfs      ;si = offset to restore data from
        mov     ax,LoadSeg
        mov     ds,ax           ;es = segment to restore data from

        call    SetGraph        ;Find our graphics segment

        mov     dx,0            ;Start at y=0
LodMn2: mov     cx,0            ;Start at x=0

LodMn1: call    PixelPos        ;Find the position of the pixel
        mov     di,bx           ;Get the index of the pixel to write

        mov     al,ds:[si]      ;Get the packed data from the buffer
        inc     si              ;  bump up pointer
        mov     ah,al           ;Copy to ah
        sar     al,4            ;Rotate it into position
        and     al,0Fh          ;Mask off unused portion of each pixel
        and     ah,0Fh
        mov     es:[di],al      ;Save to the screen
        inc     di              ;  bump up pointer
        mov     es:[di],ah
        inc     di              ;  bump up pointer

        mov     al,ds:[si]      ;Get the packed data from the buffer
        inc     si              ;  bump up pointer
        mov     ah,al           ;Copy to ah
        sar     al,4            ;Rotate it into position
        and     al,0Fh          ;Mask off unused portion of each pixel
        and     ah,0Fh
        mov     es:[di],al      ;Save to the screen
        inc     di              ;  bump up pointer
        mov     es:[di],ah

        add     cx,4            ;update 'x'
        cmp     cx,320          ;Test if we have finished one line yet
        jne     LodMn1          ;Loop until a line is finished

        inc     dx              ;We have transferred one line, update 'y'
        cmp     dx,200          ;Test if we have finished the required 200 lines
        jne     LodMn2          ;Loop until the whole transfer is complete

        pop     es
        pop     bp
        pop     ds
        ret     4
_LOADMENU endp

        db      0
;********************************************
_VLINE proc NEAR                            ;
; Draws a vertical line between 2 points    ;
;(vlx,vly1) and (vlx,vly2)                  ;
;********************************************
vlx     equ     [bp+10]
vly1    equ     [bp+8]
vly2    equ     [bp+6]
vlcolor equ     [bp+4]

        push    bp
        mov     bp,sp
        push    di

        call    OurCod3         ;Push address label OurCod5 onto stack
OurCod3: pop    di              ;Get the value back into si
        sub  di,offset(OurCod3) ;Adjust it to work out start of our code area

                                ;Test if Vline outside of 0..MaxX
        mov     cx,vlx          ;Get cx =x
        or      cx,0            ;Test if x < 0
        js      Voutside        ;  and discard draw if x <  0
        cmp     cx,MaxX         ;Test if x >= MaxX
        jnb     Voutside        ;  and discard draw if x >= MaxX

        mov     ax,vly1         ;Get the lower value y co-ordinate
        cmp     ax,MaxY         ;if y1 > MaxY then exit
        jnb     Voutside
        mov     dx,vly2         ;Get the higher value y co-ordinate
        or      dx,0            ;if y2 < 0 then exit
        jns     VCoordsOK
Voutside:
        jmp     vfinish

VCoordsOK:
        Call    SetGraph
        cmp     dx,MaxY         ;Test if larger y co-ordinate is off screen
        jb      VLin1           ;Jump if on screen
        mov     dx,MaxY         ;  else limit it to edge of screen
VLin1:
        mov     ax,vly1         ;Get the smaller y co-ordinate value again
        cmp     ax,0            ;Test if the smaller y co-ordinate is off screen
        jns     VLin2           ;  else limit it to edge of screen
        mov     ax,0
VLin2:
        mov     si,dx           ;Get the larger y co-ordinate value
        sub     si,ax           ;Subtract the smaller y co-ordinate value
        jne     VNotZero        ;Jump if there is atleast one pixel to draw
        inc     si              ;Ensure there is atleast one pixel to draw
VNotZero:
        dec     dx              ;Move x pointer up one pixel, match protelVGA640
        mov     ax,vlcolor      ;Get the pixel colour
                                ;Draw (si) pixels of colour (ax) from (dx) up
        and  ax,8000h+MaxColors ;Limit to available colours, Test for XOR op
        js      VLin5           ;Do XOR operation instead of straight write

VLin3:  call    PixelPos        ;Translate to pixel address, set window
VLin4:  mov     es:0[bx],al     ;Set the pixel to the required colour

        dec     si              ;Count down the number of pixels
        jz      vfinish         ;Exit if we have completed the line

        dec     dx              ;Move x pointer up one pixel
       sub   bx,cs:ByPerLL[di]  ;Move addr up 1 line: Subtact # pix in scan line
        jnb     VLin4           ;If we have not wrapped the video page, continue
        jmp     VLin3           ;  else re-calculate the video page address

                                ;This code exactly like VLin3 - only XOR...
VLin5:  call    PixelPos        ;Translate to pixel address, set window
VLin6:  xor     es:0[bx],al     ;Set the pixel to the required colour

        dec     si              ;Count down the number of pixels
        jz      vfinish         ;Exit if we have completed the line

        dec     dx              ;Move x pointer up one pixel
        sub  bx,cs:ByPerLL[di]  ;Move addr up 1 line: Subtact # pix in scan line
        jnb     VLin6           ;If we have not wrapped the video page, continue
        jmp     VLin5           ;  else re-calculate the video page address

vfinish:
        pop     di
        pop     bp
        ret     8
_VLINE ENDP

        db      0
;*******************************************;
_HLINE proc NEAR                            ;
; Draws a horizontal line between 2 points  ;
;(hlx1,hly) and (hlx2,hly)                  ;
;*******************************************;
hlx1    equ     [bp+10]
hlx2    equ     [bp+8]
hly     equ     [bp+6]
hlcolor equ     [bp+4]

        push    bp
        mov     bp,sp
                                ;First Check if Hline OutSide of 0..Maxy
        Mov     dx,hly          ;Get dx = y
        or      dx,0            ;Test if y < 0
        js      Houtside        ;  and discard draw if y <  0
        cmp     dx,MaxY         ;Test if y >= MaxY
        jnb     Houtside        ;  and discard draw if y >= MaxY

        Mov     ax,hlx1         ;Get the smaller x co-ordinate value for testing
        cmp     ax,MaxX         ;if x1 > MaxX then exit
        jnb     Houtside
        Mov     cx,hlx2         ;Get the larger x co-ordinate value to cx
        or      cx,0            ;if x2 < 0 then exit;
        jns     CoordsOK
HoutSide:
        jmp     hfinish

CoordsOK:
        Call    SetGraph
        cmp     cx,MaxX         ;Test if larger x co-ordinate is off screen
        jb      HLin1           ;Jump if on screen
        mov     cx,MaxX-1       ;  else limit it to edge of screen
HLin1:
        mov     ax,hlx1         ;Get the smaller x co-ordinate value again
        cmp     ax,0            ;Test if the smaller x co-ordinate is off screen
        jns     HLin2           ;  else limit it to edge of screen
        mov     ax,0
HLin2:
        mov     si,cx           ;Get the larger x co-ordinate value
        sub     si,ax           ;Subtract the smaller x co-ordinate value
                                ;When next line is commented, operation matches
                                ;  that of the original protel VGA640 driver
                                ;  (seems we have to copy a bug in their code!)
;       jne     HNotZero        ;Jump if there is atleast one pixel to draw
        inc     si              ;Ensure there is atleast one pixel to draw
HNotZero:
        mov     ax,hlcolor      ;Get the pixel colour
                                ;Draw (si) pixels of colour (ax) from (cx) left
        and  ax,8000h+MaxColors ;Limit to available colours, Test for XOR op
        js      HLin5           ;Do XOR operation instead of straight write

HLin3:  call    PixelPos        ;Translate x and y to pixel address, set window
HLin4:  mov     es:0[bx],al     ;Set the pixel to the required colour

        dec     si              ;Count down the number of pixels
        jz      hfinish         ;Exit if we have completed the line

        dec     cx              ;Move x pointer left one pixel
        sub     bx,1            ;Move address down one location
        jnb     HLin4           ;If we have not wrapped the video page, continue
        jmp     HLin3           ;  else re-calculate the video page address

                                ;This code exactly like HLin3 - only XOR...
HLin5:  call    PixelPos        ;Translate x and y to pixel address, set window
HLin6:  xor     es:0[bx],al     ;Set the pixel to the required colour

        dec     si              ;Count down the number of pixels
        jz      hfinish         ;Exit if we have completed the line

        dec     cx              ;Move x pointer left one pixel
        sub     bx,1            ;Move address down one location
        jnb     HLin6           ;If we have not wrapped the video page, continue
        jmp     HLin5           ;  else re-calculate the video page address

Hfinish:
        pop     bp
        ret     8
_HLINE ENDP


;******************************************;
_ARROWCURSOR proc NEAR                     ;
; Draws a cursor on the screen at location ;
; (acx,acy)                                ;
;******************************************;
acx   equ      [bp+6]
acy   equ      [bp+4]
      push     bp
      mov      bp,sp

      Call     SetGraph

      mov      si,0ffffh               ;15 is xor color

      mov     cx,acx                   ;get x
      mov     dx,acy                   ;get y

      mov      bx,1                    ;y := 0
      call     ArrowPoints

      inc      dx                      ;y := 1
      mov      bx,2
      call     ArrowPoints

      inc      dx                      ;y := 2
      mov      bx,3
      call     ArrowPoints

      inc      dx                      ;y := 3
      mov      bx,4
      call     ArrowPoints

      inc      dx                      ;y := 4
      mov      bx,5
      call     ArrowPoints

      inc      dx                      ;y := 5
      mov      bx,6
      call     ArrowPoints

      inc      dx                      ;y := 6
      mov      bx,7
      call     ArrowPoints

      inc      dx                      ;y := 7
      mov      bx,8
      call     ArrowPoints

      inc      dx                      ;y := 8
      mov      bx,9
      call     ArrowPoints

      inc      dx                      ;y := 9
      mov      bx,10
      call     ArrowPoints

      inc      dx                      ;y := 10
      mov      bx,11
      call     ArrowPoints

      inc      dx                      ;y := 11
      mov      bx,12
      call     ArrowPoints

      inc      dx                      ;y := 12
      mov      bx,6
      call     ArrowPoints

      inc      dx                      ;y := 13
      mov      bx,5
      call     ArrowPoints

      inc      dx                      ;y := 14
      mov      bx,4
      call     ArrowPoints

      inc      dx                      ;y := 15
      mov      bx,3
      call     ArrowPoints

      inc      dx                      ;y := 16
      mov      bx,2
      call     ArrowPoints

      inc      dx                      ;y := 17
      call     Point                   ;x := 0

      pop     bp
      ret     4
_ARROWCURSOR endp


;*****************************************
ArrowPoints Proc Near
; Sub_Routine for ArrowCursor
; X = cx
; Yoffset = Dx
; bx = count of x
;*****************************************
      Push    cx
Ploop:
      call    Point
      inc     cx       ;next x
      dec     bx
      jnz     Ploop
      pop     cx
      ret
ArrowPoints endp


;********************************************************;
_SETBACKGROUND Proc Near                                 ;
; Sets the current background color to BackColor         ;
; Procedure SetBackGround(BackColor,ColorSet : integer); ;
;********************************************************;
BackColor equ      [bp+6]
ColorSet  equ      [bp+4]
      push     bp
      mov      bp,sp
      mov      ax,BackColor
      cmp      ax,0             ;Test if the color number is 0 ... to be black
      jne      SwopBlack        ;Jump past replace unless the color number is 0
      mov      ax,16            ;Replace white (0) with black (16) background
SwopBlack:
      jmp      EndBack          ;Skip code for use with 64 colours, 12 Sept 00
;----                          ;From here to EndBack: needed only if colours<>63
      and      ax,15
      cmp      ax,6
      jne      Not6                    ;if color = 6 then color := 20;
      mov      ax,20
      jmp      EndBack
Not6:
      and      ax,8                    ;if (color > 15) then color := color + 48;
      jz       Less15
      mov      ax,backcolor
      and      ax,15
      add      ax,48
      jmp      EndBack

Less15:
      mov      ax,backcolor
      and      ax,15

EndBack:
;----
        push    ax
        call    Capabilities    ;Get our capabilities
        and     ax,00E00h       ;Test if Vesa Bios Extensions version 2 or 3
        jz      DoVGAPall       ;If less than vers 2, use VGA pallete function

                                ;For VBE 2 and above, use pallete func 09
        pop     dx              ;Get the colour to read

        call    SetupForLEA     ;Find offset, load segment to ES
        lea     di,PallWork     ;Get the address of the Pallette register
        add     di,bx           ;  and fix up the offset
                                ;es:di now point to Pallette working register

        mov     cx,1            ;1 pallette item only
        mov     bl,1            ;Get the pallette item
        mov     ax,4F09h
        int     10h

        mov     dx,0            ;Write the colour read over colour 0
        call    SetupForLEA     ;Find offset, load segment to ES
        lea     di,PallWork     ;Get the address of the Pallette register
        add     di,bx           ;  and fix up the offset
                                ;es:di now point to Pallette working register

        mov     cx,1            ;1 pallette item only
        mov     bl,0            ;Set the pallette item
        mov     ax,4F09h
        int     10h

        pop     bp
        ret     4

DoVGAPall:
        pop     bx              ;Get the colour to read
        mov     ax,1015h        ;Read single 18 bit colour register function
        int     10h             ;Read the RGB value (R dh  G ch  B cl, all 0-63)

        mov     bx,0            ;Now write the value read over colour 0
        mov     ax,1010h        ;Set single 18 bit colour register function
        int     10h
        pop      bp
        ret 4
_SETBACKGROUND endp


;*****************************************
_SETCOLORSET Proc Near
; Selects the Color set in Cset
;*****************************************
Cset equ [bp+4]
      ret 2
_SETCOLORSET endp

;*****************************************
TextPoint Proc Near
; X = Cx
; Y = Dx
; Color in SI
;*****************************************
        cmp     dx,MaxY         ;If y>lines-on-screen then don't draw it
        jb      Point2          ;  ..if it's smaller go do rest of scope test
        jmp     Poutside        ;  ..if it's larger, exit

TEXTPOINT  ENDP

;*****************************************
Point Proc Near
; X = Cx
; Y = Dx
; SI = color                                 ;
; Internal procedure called by Draw,PlotPixel;
; and others to set a pixel to given color.  ;
;*****************************************
        cmp dx,MaxY-StatusLine  ;if y bigger then no carry
        jnb     Poutside        ;Jump if carry not set
Point2:
        or      cx,0            ;Test if x is negative
        js      Poutside
        or      dx,0            ;Test if y is negative
        js      Poutside
        cmp     cx,MaxX         ;if x bigger then no carry
        jnb     Poutside        ;Jump if carry not set

        push    ax
        push    bx

        call    PixelPos        ;Go find the pixel position
        mov     ax,si           ;Get the pixel colour
        and  ax,8000h+MaxColors ;Limit to available colours, Test for XOR op
        jns     Point3          ;Jump if just standard write
        xor     es:0[bx],al     ;Do XOR operation instead of straight write
        jmp     Point4

Point3: mov     es:0[bx],al     ;Set the pixel to the required colour

Point4: pop     bx
        pop     ax
Poutside:
        ret
POINT   ENDP

;*****************************************
PixelPos Proc Near
; X = Cx
; Y = Dx
; Internal procedure checks if the pixel x,y
; is inside the current window, if it is not
; then the current window is readjusted so
; that the pixel is inside
; Return Bx = offset to pixel in window
;*****************************************
        push    ax
        push    dx
        push    si
        call    OurCod5         ;Push address label OurCod5 onto stack
OurCod5: pop    si              ;Get the value back into si
        sub  si,offset(OurCod5) ;Adjust it to work out start of our code area

        mov   ax,cs:ByPerLL[si] ;Number pixels in a scan line
        mul     dx              ;Multiplied by 'y' value
        add     ax,cx           ;  and then added to 'x' gives abs pixel postion
        adc     dx,0            ;  (Propagate carry to high word)
        push    ax              ;Save the low word of the absolute pixel postion

        cmp   dx,cs:CurrWin[si] ;Test high word (window) against CurrWin setting
        je      PixelPosOut     ;Exit if we are in the correct window already
        mov   cs:CurrWin[si],dx ;Save the new CurrWin setting
                                ;dx contains the desired 64k page window

        mov     ax,64           ;Number of kilobytes in a '64k page'
        mul     dx              ;Number of kilobytes to base of this page
                                ;  result is in dx:ax
        mov   bx,cs:WinGran[si] ;Get the window granularity
        or      bx,bx           ;Test it - we expect it to be 64
        je      PixelPosOut     ;If it is zero, don't divide!
        div     bx              ;Divide by the window granularity
                                ;Result in ax is window position in mem

        mov     dx,ax           ;Move window position to dx

        mov     bx,cs:PageFun[si]       ;Get first word of Graphics window call
        or      bx,cs:PageFun+2[si]     ;OR with next half of call address
        mov     ax,4F05h                ;Code: Set SVGA Memory window position
        jnz     PixelP2                 ;Jump if there is a valid address

PixelP1:                                ;Use BIOS call for position - a bit slow
        push    dx                      ;Save the desired window position
        mov     bx,0                    ;Use BIOS call, not direct FAR CALL
        int     10h
        pop     dx                      ;Restore the desired window position
        mov     bx,1                    ;Repeat for window 1
        mov     ax,4F05h                ;Code: Set SVGA Memory window position
        int     10h
        jmp     PixelPosOut

PixelP2:                                ;Use direct call for position - faster
        push    dx                      ;Save the desired window position
        mov     bx,0
        call  DWORD PTR cs:PageFun[si]  ;Set graphics window to value in dx
        pop     dx                      ;Restore the desired window position
        mov     bx,1                    ;Repeat exercise for window 1
        mov     ax,4F05h                ;Code: Set SVGA Memory window position
        call  DWORD PTR cs:PageFun[si]  ;Set graphics window to value in dx

PixelPosOut:
        pop     bx              ;Restore return value - low word pixel position
        pop     si
        pop     dx
        pop     ax
        ret
PixelPos endp


;*****************************************
GTFGen   Proc Near
; Internal procedure to generate timing
; values for programming the CRT controller
;*****************************************
;        ret                     ;Exit until code is finished and debugged
        call    SetupForLEA     ;Setup segment and offset

        mov ax,es:PixelClock+2[bx] ;Get the high word of current pixel clock
        sal ax,4                   ;Divide by 4096, discarding MSBs
        mov cx,es:PixelClock[bx]   ;Get the low word of current pixel clock
        shr cx,12                  ;Divide by 4096, discarding MSBs
        or  ax,cx                  ;Result is PixelClock*4096, 16 bit value
                                   ;  Range 0-268Mhz in jumps of 4096Hz
        mov es:HClk[bx],ax         ;Save the HClk value

        mov es:HTotal[bx],(MaxX/16)+2 ;Init the HTotal value
TryHBlank:
        mov ax,es:HCLK[bx]      ;Get PixelClock*4096
        div es:HTotal[bx]       ;Divide by HTotal to give HSyncRate * (4096/16)
        sal ax,4                ;Multiply by 16: Gives    HSyncRate * (4096/256)
        mov cx,ax               ;Save high order result
        mov ax,dx               ;Get Remainder
        sal ax,4                ;Multiply by 16 for second stage division
        div es:HTotal[bx]       ;Second stage of division
        add cx,ax               ;Result is 625...8176  for  10khz...130.816khz

        mov ax,47998            ;Magic number to get result range: 19660 = 100us
        div cx                  ;Calculate period: Result in units of 1.3021us
        push ax                 ;Save High order result
        mov ax,dx               ;Get remainder
        sal ax,4                ;Multiply by 16 for second stage division
        div cx                  ;Calculate period: Result in units of 81.381ns
                                ;It follows that the ax result is less than 16..
        sal ax,4                ;Multiply by 16: Units now 5.08ns, accuracy 81ns
        pop dx                  ;Restore high order result
        sal dx,8                ;Multiply by 256
        add ax,dx               ;Combine high/low order results
       mov es:HTotalTime[bx],ax ;Save the calculated time

                                ;Range of ax should be 0-19660 (=0%..30%/100us)
        add ax,45874            ;Add the equivalent of 70% to the ratio
        jc HaveHTotalVal        ;If wrapping ratio, exit the compare loop
                                ;Here with trial HPix/HTotal ratio in ax
        mul es:HTotal[bx]       ;Scale HTotal by ratio to see if enough HBlank
                                ;Result in DX is tentative MaxX/16 value
        cmp dx,MaxX/16          ;Test: Is HTotal and HBlank large enough yet?
        jge HaveHTotalVal       ;Jump if Success! We have a large enough HTotal
                                ;  ...or no, we need HTotal to go still larger
        inc es:HTotal[bx]       ;Bump up HTotal
        cmp es:HTotal[bx],255   ;Scope test HTotal
        jl TryHBlank            ;Loop until we have found our HBlank value
;       jmp HaveHTotalVal

HaveHTotalVal:
        mov ax,16
        mul es:HTotal[bx]       ;Multiply HTotal by 16
        mov es:HorzTotal[bx],ax ;Save the calculated Horizontal total value
        mov dx,200
        div dx                  ;then divide by 200 - equiv to Divide by 12.5
                                ;Result in ax is HSYNC PulseWidth in 16cyc lumps
        mov dx,es:HTotal[bx]    ;Get the HTotal value in 16 pixel lumps
        sub dx,MaxX/16          ;Subtract the active pixels screen width
        sub dx,ax               ;Subtract the HSync width
                                ;Result is each of 2 HPorch vals in 8 pix lumps!
        add dx,MaxX/8           ;Add the active pixels screen width
                                ;Result is HorzSyncStart value in 8 pix lumps
        sal dx,3                ;Multiply to get HorzSyncStart in pixel units
    mov es:HorzSyncStart[bx],dx ;Save the calculated HSync Start value
        sal ax,4                ;Multiply to get HorzSync PulseWidth in pixels
        add ax,dx               ;Result is calculated HSync end value
      mov es:HorzSyncEnd[bx],ax ;Save the value

                                ;Horizontal vals done, now update the Vertical
       mov dx,es:HTotalTime[bx] ;Get the calculated Horizontal line time
        shr dx,1                ;Divide by 2: Make units = 10.16ns
        mov ax,54133            ;Desire VSYNC+VBackPorch= 550us in 10.16ns units
        div dx                  ;Calculate the number of lines in VBlnk+VBkPorch
        add dx,MaxY+2           ;Calculate total number of lines on screen
        mov es:VertTotal[bx],dx ;Save total number of lines on screen
  mov es:VertSyncStart[bx],MaxY+1 ;Value for Sync start
  mov es:VertSyncEnd[bx],MaxY+4  ;Value for Sync end

                                 ;Just the Vertical refresh rate left to do...
        mov ax,es:HClk[bx]       ;Get PixelClock * 4096
        div es:HTotal[bx]        ;Divide by HTotal, gives HSyncRate * 256
        sal ax,6                 ;Multiply by 64 - gives HSyncRate * 4
        mov cx,ax                ;Save high order result
        mov ax,dx                ;Get remainder
        sal ax,6                 ;Multiply by 64 - again, for HSyncRate * 4
        or  ax,1                 ;Ensure no divide by zero
        div es:HTotal[bx]        ;Execute second half of division
        add ax,cx                ;Combine - now have HSyncRate * 4

        div es:VertTotal[bx]     ;Divide Horizontal refresh by Vertical total
        sal ax,6                 ;Multiply result by 64
        mov cx,ax                ;Save high order result
        mov ax,dx                ;Get remainder
        sal ax,6                 ;Multiply by 64
        or  ax,1                 ;Ensure no divide by zero
        div es:VertTotal[bx]     ;Second stage of division
        add ax,cx                ;Combine - now have HSyncRate in 1/16Hz units

        mov dx,1600              ;Convert to units of 1/25600Hz
        mul dx
        mov al,ah                ;Get low order result to al
        mov ah,dl                ;Get high order result to ah
                                 ;Result is in units of 1/100th Hz
       mov es:RefreshRate[bx],ax ;Save the calculated refresh rate (whew!)
       mov es:CRTCFlags[bx],0    ;Set flags to off
        ret
GTFGen endp


FailMsg db 'HW runs Vesa VBE version X; DRIVER FAILED, video mode not supported'
db      0Ah,0Dh

ProgEnd = $ - GETDRIVERTYPE
ParamStart = ((ProgEnd+16)/16)*16)
db (ParamStart-ProgEnd) DUP(0)
CurrWin        dw       0FEDCh          ;Storage location for 'current window'
Capability     dw       0
PallWork       dd       0
HClk           dw       0
HTotal         dw       (MaxX/16)+2
HTotalTime     dw       0
db 6 dup(0)

; 1280x1024@61Hz, Non-Interlaced mode
; Horizontal Sync=64.25kHz
; Timing: H=(0.44us, 1.67us, 1.82us) V=(0.02ms, 0.05ms, 0.41ms)
;
; name        clock   horizontal timing     vertical timing      flags
; "1280x1024"  110     1280 1328 1512 1712   1024 1025 1028 1054

;CRTCInfoBlock:
;HorzTotal       dw      1712
;HorzSyncStart   dw      1328
;HorzSyncEnd     dw      1512
;VertTotal       dw      1054
;VertSyncStart   dw      1025
;VertSyncEnd     dw      1028
;CRTCFlags       db      0
;PixelClock      dd      110000000
;RefreshRate     dw      6096
;db 41 dup(0)


; VESA 800x600@56Hz Non-Interlaced mode
; Horizontal Sync = 35.1kHz
; Timing: H=(0.67us, 2.00us, 3.56us) V=(0.03ms, 0.063ms, 0.70ms)
;
; name        clock   horizontal timing     vertical timing      flags
; "800x600"     36      800  824  896 1024    600  601  603  625

CRTCInfoBlock:
HorzTotal       dw      1024
HorzSyncStart   dw      824
HorzSyncEnd     dw      896
VertTotal       dw      625
VertSyncStart   dw      601
VertSyncEnd     dw      603
CRTCFlags       db      0
PixelClock      dd      38000000
RefreshRate     dw      6096
db 41 dup(0)


VesaBuf:                        ;Vesa SVGA video mode info placed here
ModeAtt         dw      0
WinAttA         db      0
WinAttB         db      0
WinGran         dw      0
WinSize         dw      0
WinSegA         dw      0
WinSegB         dw      0
PageFun         dw      0,0
ByPerLL         dw      0
XRes            dw      1
YRes            dw      1
db 238 DUP(0)

VBEInfoBlock:
db 'VBE2'
VBEVers         dw      0
OEMStringPTR    dd      0
VBECapable      dw      4 DUP(0)
VideoModePTR    dd      0
TotalMemory     dw      0


db ' This file is a Protel Autotrax 1.61nd / schematic V3.31 VESA video driver. '
;db 'With some VESA cards the mode number may change for newer display resolutions. '
db 'Mode number is in text near the start of the file and appears in the '
db 'binary LSB first, so mode 107h appears as 07 01 '
db 'in the last version at offsets 0614h,0627h,065Ah,0674h. '
;db 'Make sure you have the lastest VESA driver for your video card. '
;db 'strange problems (eg garbage on screen when alt-tabbing) can be fixed '
;db 'by updating your VIDEO CARD VESA DRIVER ( - different from this Driver!) '
db 'To use this driver, copy it as GRAPH.DRV into c:\autotrax '
db 'This driver is released into the public domain. '
;db 'and you can do what you like with it with no obligation. '
db 'Go to www.airborn.com.au/layout/easytrax.html for latest version. '
db 'This version is 1.14a '
;db 'Since there was a coding improvement and no real impact on performance '
;db 'this driver runs in a 256 colour mode, although it only uses 16 colours. '
;db 'Please email if you write driver improvements, or other autotrax add-ons. '
;db 'steve(atsymbol)airborn.com.au '

db 0Dh,0Ah,20h

db '...I do not know whether I was then a man dreaming I was a butterfly, '
db 'or whether I am now a butterfly dreaming I am a man.  - Chuang Tzu '

; Have the assembler produce a fixed 4096 byte size Binary file.

CodeLength = $ - GETDRIVERTYPE
db 4086-CodeLength DUP(0)

CODE  ends
.386
CODE  SEGMENT BYTE PUBLIC
      ASSUME CS:CODE

public  MOVEECX      ;386 CODE TO MANAGE ECX REGISTER
MOVEECX Proc Near
        pop     ecx
        int     10h
        push    ecx
        jmp     RETURNECX
db      0
MOVEECX endp

CODE  ends
      end


